{$A+,B-,D+,E-,F-,G+,I-,L+,N-,O-,P-,Q-,R-,S-,T-,V-,X+}
{$M 16384,0,90000}

{       DIZEXT - FILE_ID.DIZ extractor          }
{       Coded by Mad Max / Queue Members Group  }
{       Contact me on 2:5057/14@fidonet         }

program DIZEXT;

uses Objects, DOS, Drivers
{$IFDEF OS2}
,Use32,Os2Base,Os2Def
{$ENDIF}
;

type
  TChoice = (Yes, No, Used);

const

  MINOR_VERSION    = 51;
  MAJOR_VERSION    = 1;

  FILES_BBS  = 'FILES.BBS';
  FILEID_DIZ = 'FILE_ID.DIZ';
  TEMP_FILE  = '$DIZEXT$.TMP';
  DefaultDescription: String  = 'No description found';
  KillMissed: Boolean = False;
  LogSkipped: Boolean = True;
  LogMissed:  Boolean = True;
  KillDsc: TChoice    = No;
  KillTics: TChoice   = No;
  DownloadCounter: String = '[00]';
  MaxLines: Integer = MaxInt;

type
  TStrCollection = object(TCollection)
    procedure FreeItem(Item: Pointer); virtual;
  end;

procedure TStrCollection.FreeItem(Item: Pointer);
begin
  DisposeStr(Item);
end;

type
  PArcRec = ^TArcRec;
  TArcRec = object( TObject )
    Extension:  String[3];
    Command:    String;
    constructor Init( Ext, Com: String );
  end;

constructor TArcRec.Init( Ext, Com: String );
begin
  inherited Init;
  Extension := Ext;
  Command := Com;
end;

type
  PTicRec = ^TTicRec;
  TTicRec = object( TObject )
    Used:       Boolean;
    SelfName:   String[12];
    FileName:   String[12];
    Desc:       String;
    constructor Init( _SelfName, _FileName, _Desc: String );
  end;

constructor TTicRec.Init( _SelfName, _FileName, _Desc: String );
begin
  inherited Init;
  SelfName := _SelfName;
  FileName := _FileName;
  Desc := _Desc;
end;

type
  PTicCollection = ^TTicCollection;
  TTicCollection = object(TSortedCollection)
    function Compare(Key1, Key2: Pointer): Integer; virtual;
    function KeyOf(Item: Pointer): Pointer; virtual;
  end;

function TTicCollection.KeyOf(Item: Pointer): Pointer;
begin
  KeyOf := @(PTicRec(Item)^.FileName);
end;

function TTicCollection.Compare(Key1, Key2: Pointer): Integer;
begin
  Compare := 0;
  if PString(Key1)^ > PString(Key2)^ then Compare := 1;
  if PString(Key1)^ < PString(Key2)^ then Compare := -1;
end;

var
  Areas:                TStringCollection;
  Dsc,
  Tics:                 TTicCollection;
  Archivers:            TCollection;
  F:                    TBufStream;
  DscDir,
  TicDir,
  ArcList,
  AreaList:             String;

const
  Spaces = [' ',#9,#10,#13,':',','];

function GetToken( var F:TStream ): String;
var
  S:    String;
  C:    Char;
begin
  S := '';
  repeat F.Read( C, 1 ) until not (C in Spaces) or (F.Status<>stOk);
  repeat
    S := S + UpCase(C);
    F.Read( C, 1 )
  until (C in Spaces) or (F.Status<>stOk);
  GetToken := S;
end;

function GetLine( var F:TStream; ToUp: Boolean ): String;
var
  S:    String;
  C:    Char;
begin
  S := '';
  C := #0;
  repeat F.Read( C, 1 ) until not (C in Spaces) or (F.Status<>stOk);
  while (C <> #13) and (F.Status=stOk) do
  begin
    if ToUp then
      S := S + UpCase(C)
    else
      S := S + C;
    F.Read( C, 1 )
  end;
  while S[ Length(S) ] = ' ' do Dec(S[0]);
  GetLine := S;
end;

procedure ExpandPath( var FileName: String );
var
  Dir:          DirStr;
  Name:         NameStr;
  Ext:          ExtStr;
  S:            String;
begin
  FSplit( FileName, Dir, Name, Ext );
  if Dir = '' then
  begin
    S := ParamStr(0);
    FSplit( S, Dir, Name, Ext );
    FileName := Dir + FileName;
  end;
end;

procedure ParseConfig( ConfigName: String );
var
  S:            String;
  Tmp,
  Dummy:        Integer;
begin
  F.Init( ConfigName, stOpenRead, 4096 );
  repeat
    S := GetToken(F);
    if S = ';' then
    begin
      S := GetLine(F,True);
      Continue;
    end;
    if S = 'TICDIR' then
      TicDir := GetLine(F,True);
    if S = 'DSCDIR' then
      DscDir := GetLine(F,True);
    if S = 'AREALIST' then
    begin
      AreaList := GetLine(F,True);
      ExpandPath( AreaList );
    end;
    if S = 'ARCLIST' then
    begin
      ArcList := GetLine(F,True);
      ExpandPath( ArcList );
    end;
    if S = 'KILLTICS' then
    begin
     S := GetToken(F);
     if S = 'YES' then KillTics := Yes;
     if S = 'USED' then KillTics := Used;
    end;
    if S = 'KILLDSC' then
    begin
     S := GetToken(F);
     if S = 'YES' then KillDsc := Yes;
     if S = 'USED' then KillDsc := Used;
    end;
    if S = 'LOGSKIPPED' then
    begin
     S := GetToken(F);
     if S = 'NO' then LogSkipped := False;
    end;
    if S = 'KILLMISSED' then
    begin
     S := GetToken(F);
     if S = 'YES' then KillMissed := True;
    end;
    if S = 'LOGMISSED' then
    begin
     S := GetToken(F);
     if S = 'NO' then LogMissed := False;
    end;
    if S = 'DOWNLOADCOUNTER' then
      DownloadCounter := GetToken(F);
    if S = 'DEFAULTDESCRIPTION' then
      DefaultDescription := GetLine(F,False);
    if S = 'MAXDESCRIPTIONLINES' then
    begin
      S := GetLine(F,False);
      Val( S, Tmp, Dummy );
      if Tmp > 0 then MaxLines := Tmp;
    end;
  until F.Status <> stOk;
  F.Done;
end;

procedure ProcessTics;
var
  T:    Text;
  SR:   SearchRec;
  Name,
  Desc: String;
  S:    String;
begin
  if TicDir[ Length(TicDir) ] <> '\' then
    TicDir := TicDir + '\';
  FindFirst( TicDir+'*.TIC', 0, SR );
  while Dos.DosError = 0 do
  begin
    F.Init(TicDir+SR.Name,stOpenRead,4096);
    repeat
      S := GetToken(F);
      if S = 'DESC' then
        Desc := GetLine(F,False);
      if S = 'FILE' then
        Name := GetToken(F);
    until F.Status <> stOk;
    Tics.Insert( New( PTicRec, Init( SR.Name, Name, Desc ) ) );
    F.Done;
    FindNext(SR);
  end;
{$IFDEF OS2}
    DosFindClose(SR.HDir);
{$ENDIF}
end;

procedure ProcessDsc( StartDir: String );
var
  T:    Text;
  SR:   SearchRec;
  Desc,
  S,
  Name: String;
  D:    DirStr;
  E:    ExtStr;
begin
  if StartDir[ Length(StartDir) ] <> '\' then
    StartDir := StartDir + '\';
  FindFirst( StartDir+'*.DSC', 0, SR );
  while Dos.DosError = 0 do
  begin
    F.Init(StartDir+SR.Name,stOpenRead,4096);
    Desc := ''; Name := '';
    repeat
      S := GetToken(F);
      if (Name='') and (S = 'OUT') and (GetToken(F) = 'FILE') then
      begin
        S := GetLine(F,True);
        FSplit( S, D, Name, E );
        Name := Name+E;
      end;
      if S = 'SUBJ' then
        Desc := GetLine(F,False);
    until (F.Status <> stOk) or (Desc<>'');
    Dsc.Insert( New( PTicRec, Init( SR.Name, Name, Desc ) ) );
    F.Done;
    FindNext(SR);
  end;
{$IFDEF OS2}
  DosFindClose(SR.HDir);
{$ENDIF}
  FindFirst( StartDir+'*.*', Directory, SR );
  while Dos.DosError = 0 do
  begin
    if (SR.Name <> '.') and (SR.Name <> '..') and
       (SR.Attr and Directory > 0 ) then
    begin
      ProcessDsc( StartDir+SR.Name );
    end;
    FindNext(SR);
  end;
{$IFDEF OS2}
  DosFindClose(SR.HDir);
{$ENDIF}
end;

{$IFNDEF OS2}
procedure DosDelete( Name: PChar );
begin
  asm
    push ds
    push ss
    pop ds
    lea dx,Name
    mov ah,41h
    int 21h
    pop ds
  end;
end;
{$ENDIF}

procedure DoKillTics( How: TChoice );
var
  SR:   SearchRec;
  S:    String;
  I:    Integer;
begin
  WriteLn( 'Deleting TIC''s' );
  if TicDir[ Length(TicDir) ] <> '\' then
    TicDir := TicDir + '\';
  if How = Used then
  begin
    for I := 1 to Tics.Count do
    begin
      with TTicRec(Tics.At(I-1)^) do
      begin
        if Used then
        begin
          S := TicDir+SelfName;
          S[ Length(S)+1 ] := #0;
          DosDelete(@S[1]);
        end;
      end;
    end;
  end
  else
  begin
    FindFirst( TicDir+'*.TIC', 0, SR );
    while Dos.DosError = 0 do
    begin
      S := TicDir+SR.Name;
      S[ Length(S)+1 ] := #0;
      DosDelete(@S[1]);
      FindNext(SR);
    end;
{$IFDEF OS2}
    DosFindClose(SR.HDir);
{$ENDIF}
  end;
end;

procedure DoKillDsc( StartDir: String; How: TChoice );
var
  SR:   SearchRec;
  S:    String;
  I:    Integer;
begin
  WriteLn( 'Deleting DSC''s' );
  if StartDir[ Length(StartDir) ] <> '\' then
    StartDir := StartDir + '\';
  if How = Used then
  begin
    for I := 1 to Dsc.Count do
    begin
      with TTicRec(Dsc.At(I-1)^) do
      begin
        if Used then
        begin
          S := StartDir+SelfName;
          S[ Length(S)+1 ] := #0;
          DosDelete(@S[1]);
        end;
      end;
    end;
  end
  else
  begin
    FindFirst( StartDir+'*.DSC', 0, SR );
    while Dos.DosError = 0 do
    begin
      S := StartDir+SR.Name;
      S[ Length(S)+1 ] := #0;
      DosDelete(@S[1]);
      FindNext(SR);
    end;
{$IFDEF OS2}
    DosFindClose(SR.HDir);
{$ENDIF}
  end;
  FindFirst( StartDir+'*.*', Directory, SR );
  while Dos.DosError = 0 do
  begin
    if (SR.Name <> '.') and (SR.Name <> '..') and
       (SR.Attr and Directory > 0 ) then
    begin
      DoKillDsc( StartDir+SR.Name, How );
    end;
    FindNext(SR);
  end;
{$IFDEF OS2}
  DosFindClose(SR.HDir);
{$ENDIF}
end;

procedure MakeLists;
var
  S:    String;
  SR:   SearchRec;
  Ext,
  Com:  String;
begin
  F.Init( AreaList, stOpenRead, 4096 );
  if F.Status <> stOk then
  begin
    WriteLn( 'Error : ', AreaList, ' not found' );
    Halt;
  end
  else
  begin
    repeat
      S := GetLine(F,True);
      if Pos( '*', S ) <> 0 then
      S[0] := Chr( Pos( '*', S )-2 );
      if S[ Length(S) ] = '\' then Dec(S[0]);
      FindFirst( S, Directory, SR );
      if Dos.DosError = 0 then Areas.Insert(NewStr(S));
{$IFDEF OS2}
      DosFindClose(SR.HDir);
{$ENDIF}
    until F.Status <> stOk;
  end;
  F.Done;
  F.Init( ArcList, stOpenRead, 4096 );
  if F.Status <> stOk then
  begin
    WriteLn( 'Error : ', S, ' not found' );
  end
  else
  begin
    repeat
      Ext := GetToken(F);
      Com := GetLine(F,True);
      Archivers.Insert( New( PArcRec, Init( Ext, Com )));
    until F.Status <> stOk;
  end;
  F.Done;
end;

function ToUpper( var S: String ): String;
var
  I:    Integer;
begin
  for I := 1 to Length(S) do S[I] := UpCase( S[I] );
  ToUpper := S;
end;

procedure ProcessArea( AreaDir: String );
var
  Files:        TStringCollection;
  S:            String;
  PS:           PString;
  SR:           SearchRec;
  Dummy:        Integer;
  FilesBBS:     Text;
  DIZ:          TStrCollection;

function ExtractDIZ( FileName: PathStr ): String;

procedure ExtractFromArchive( Command: String );
const
  FILE_ID : String = FILEID_DIZ+#0;
  P : PChar = @(FILE_ID[1]);
var
  S:    String;
  L:    Text;
begin
{$IFNDEF OS2}
  asm
    push ds
    lds dx,P
    mov ah,41h
    int 21h
    pop ds
  end;
{$ELSE}
  DosDelete(P);
{$ENDIF}
  Exec( GetEnv( 'COMSPEC'), ' /c '+ Command + ' '+ FileName + ' ' +
                FILEID_DIZ+' > nul' );
  if Dos.DosError <> 0 then
    WriteLn( 'DOS Error : ', Dos.DosError );
  Assign(L,FILEID_DIZ);

  Reset(L);
  if System.IOResult = 0 then
  begin
    repeat
      ReadLn(L,S);
      if S <> '' then DIZ.Insert( NewStr(S) );
    until Eof(L);
    Close(L);
  end;
end;

var
  Dir:          DirStr;
  Name:         NameStr;
  Ext:          ExtStr;
  Result:       Boolean;
  I,J:          Integer;
  S:            String;
  PS:           PString;

begin
  DIZ.FreeAll;
  I := System.IOResult;
  FSplit( FileName, Dir, Name, Ext );
  S := Name+Ext;
  if Tics.Search( @S, I ) then
  begin
    DIZ.Insert( NewStr(PTicRec(Tics.At(I))^.Desc) );
  end
  else
  begin
    FSplit( FileName, Dir, Name, Ext );
    for I := 1 to Archivers.Count do
    begin
      with TArcRec(Archivers.At(I-1)^) do
        if Copy(Ext,2,3) = Extension then
          ExtractFromArchive( Command );
    end;
  end;
  if Dsc.Search( @S, I ) and (DIZ.Count = 0) then
  begin
    DIZ.Insert( NewStr(PTicRec(Dsc.At(I))^.Desc) );
  end;
  if DIZ.Count > 0 then
  begin
    for I := 1 to DIZ.Count do
    begin
      if MaxLines < I then Break;
      if I = 1 then
      begin
        S := Name+Ext;
        PS := @S;
        FormatStr( S, '%-12s '+DownloadCounter+' ', PS );
        Write( FilesBBS, S );
        WriteLn( FilesBBS, PString(DIZ.At(I-1))^ );
      end
      else
        WriteLn( FilesBBS, '                                    '+PString(DIZ.At(I-1))^ )
    end;
    ExtractDIZ := 'Description extracted';
  end
  else
  begin
    S := Name+Ext;
    PS := @S;
    FormatStr( S, '%-12s ' + DownloadCounter + ' ' + DefaultDescription, PS );
    WriteLn( FilesBBS, S );
    ExtractDIZ := DefaultDescription;
  end;
end;

var
  Tmp:          String;
  TempFile:     Text;
  Missed:       Boolean;
  I:            Integer;

begin
  Files.Init(100,10);
  DIZ.Init(30,1);
  WriteLn( 'Processing directory ', AreaDir );
  Assign(FilesBBS,AreaDir+'\' + FILES_BBS );
  Reset(FilesBBS);
  if System.IOResult <> 0 then
    Rewrite(FilesBBS)
  else
  begin
    if KillMissed then
    begin
      Assign( TempFile, AreaDir+'\'+TEMP_FILE );
      Rewrite(TempFile);
    end;
    repeat
      ReadLn(FilesBBS,S);
      if (S[1] <> ' ') and (S<>'') then
      begin
        Tmp := Copy( S, 0,Pos( ' ', S )-1 );
        if Tmp = '' then Tmp := S;
        if (Tmp <> '            ') and (Tmp <> TEMP_FILE) then
        begin
          FindFirst( AreaDir + '\' + Tmp, 0, SR );
          if Dos.DosError = 0 then
          begin
            Files.Insert(NewStr(ToUpper(Tmp)));
            Missed := False;
          end
          else
          begin
            Missed := True;
            if LogMissed then
              WriteLn( 'Deleting missing file ''', ToUpper(Tmp), '''' );
          end;
{$IFDEF OS2}
          DosFindClose(SR.HDir);
{$ENDIF}
        end;
      end;
      if (KillMissed = True) and not Missed then
      begin
        WriteLn( TempFile, S );
      end;
    until Eof(FilesBBS);
    Close(TempFile);
    Close(FilesBBS);
    if KillMissed then
    begin
      Erase(FilesBBS);
      Rename( TempFile, AreaDir+'\'+FILES_BBS );
    end;
    Assign(FilesBBS, AreaDir+'\'+FILES_BBS);
    Append(FilesBBS);
  end;

  if System.IOResult <> 0 then Halt;

  FindFirst( AreaDir+'\*.*', 0, SR );
  while Dos.DosError = 0 do
  begin
    ToUpper(SR.Name);
    if Dsc.Search( @SR.Name, I ) then
      TTicRec(Dsc.At(I)^).Used := True;
    if Tics.Search( @SR.Name, I ) then
      TTicRec(Tics.At(I)^).Used := True;
    Tmp := SR.Name; Tmp[0] := #6;
    if Tmp <> 'FILES.' then
    begin
      PS := @SR.Name;
      FormatStr( S, '%-12s : ', PS );
      if not Files.Search(@SR.Name,Dummy) then
        WriteLn( S, ExtractDIZ( AreaDir+'\'+SR.Name ) )
      else
      begin
        if LogSkipped then
          WriteLn( S, 'Already in ', FILES_BBS, ', skipped' );
      end;
    end;
    FindNext(SR);
  end;
{$IFDEF OS2}
    DosFindClose(SR.HDir);
{$ENDIF}
  Files.Done;
  DIZ.Done;
  Close(FilesBBS);
end;

var
  I:            Integer;
  CurDir,
  TempDir:      String;
{$IFNDEF OS2}
  CurDisk:      Byte;
{$ELSE}
  Temp,
  CurDisk:      ULong;
{$ENDIF}
  S:            String;
begin
  WriteLn;
  WriteLn( 'Description Extractor V',MAJOR_VERSION,'.',MINOR_VERSION,
           '  Coded by Mad Max / Queue Members Group (c)95' );
  WriteLn( '------------------------------------------------------------------------' );
  Areas.Init(100,1);
  Archivers.Init(10,1);
  Tics.Init(10,1);
  Dsc.Init(10,1);
  AreaList := Copy( ParamStr(0), 0, Length(ParamStr(0))-3 ) + 'LST';
  ArcList := Copy( ParamStr(0), 0, Length(ParamStr(0))-3 ) + 'ARC';
  TicDir := '';
  DscDir := '';
  ParseConfig(Copy( ParamStr(0), 0, Length(ParamStr(0))-3 ) + 'CFG');
  MakeLists;
  if TicDir <> '' then
    ProcessTics;
  if DscDir <> '' then
    ProcessDsc( DscDir );
  if GetEnv( 'TEMP' ) <> '' then
  begin
{$IFNDEF OS2}
    asm
      xor dl,dl
      mov si,offset CurDir+1
      mov ah,47h
      int 21h
      mov ah,19h
      int 21h
      mov CurDisk,al
    end;
{$ELSE}
    DosQueryCurrentDisk( CurDisk, Temp );
    Temp := 255;
    DosQueryCurrentDir( CurDisk, CurDir[1], Temp );
{$ENDIF}
    TempDir := GetEnv( 'TEMP' );
    if TempDir[2] = ':' then
    begin
{$IFNDEF OS2}
      asm
        mov ah,0Eh
        mov dl,byte ptr TempDir[1]
        sub dl,'A'
        int 21h
      end;
{$ELSE}
      Temp := Ord(TempDir[1])-Ord('A');
      DosSetDefaultDisk(Temp);
{$ENDIF}
    end;
    TempDir[Length(TempDir)+1] := #0;
{$IFNDEF OS2}
    asm
      mov dx,offset TempDir+1
      mov ah,3Bh
      int 21h
    end;
{$ELSE}
    DosSetCurrentDir(@TempDir[1]);
{$ENDIF}
  end;

  for I := 1 to Areas.Count do
    ProcessArea( PString(Areas.At(I-1))^ );
  if KillTics in [Yes,Used] then DoKillTics( KillTics );
  if KillDsc in [Yes,Used] then DoKillDsc( DscDir, KillDsc );
  CurDir[0] := '\';

  if GetEnv( 'TEMP' ) <> '' then
  begin
{$IFNDEF OS2}
    asm
      mov ah,0Eh
      mov dl,CurDisk
      int 21h
      mov dx,offset CurDir
      mov ah,3Bh
      int 21h
    end;
{$ELSE}
    DosSetDefaultDisk( CurDisk );
    DosSetCurrentDir( @CurDir[1] );
{$ENDIF}
  end;
  Areas.Done;
  Archivers.Done;
  Tics.Done;
  Dsc.Done;
end.
